@@ -1,10 +1,11 @@
 from PyQt6.QtWidgets import QApplication
 from pyxy3d.gui.main import CalibrationWizard
 import sys
+from time import sleep
 from pyxy3d import __root__
 from pathlib import Path
 
-config_path = Path(__root__, "tests", "just_checking")
+config_path = Path(__root__, "tests", "217")
     
     
 app = QApplication(sys.argv)
@@ -19,6 +20,10 @@ window.wizard_directory.original_path.textbox.setText(str(config_path))
 window.wizard_directory.modified_path.textbox.setText(str(config_path))
 window.wizard_directory.launch_wizard_btn.click()
 window.wizard_charuco.navigation_bar.next_wizard_step_btn.click()
+# while not hasattr(window, "stereoframe"):
+#     sleep(1)
+# window.stereoframe.navigation_bar.calibrate_collect_btn.click()
+
 window.show()
 
 app.exec()
\ No newline at end of file
